package cn.kdan.compdf.filter;

import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.base.R;
import cn.kdan.compdf.config.JwtConfig;
import cn.kdan.compdf.config.MyRedisConstant;
import cn.kdan.compdf.dto.LoginUser;
import cn.kdan.compdf.dto.LoginUserDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.feign.UserFeign;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;

/**
 * 全局登录鉴权过滤器
 * @author Bob 2022-10-12
 **/
@Component
@Slf4j
@RequiredArgsConstructor
public class AuthGlobalFilter implements GlobalFilter, Ordered {

    @Autowired
    private StringRedisTemplate redisTemplate;

    private final UserFeign userFeign;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 1、判断放行请求地址(登录接口)

        // 2、获取请求头jwt
        String authorization = exchange.getRequest().getHeaders().getFirst(AuthEnum.AUTHORIZATION);
        // 没有jwt
        if (StringUtils.isEmpty(authorization)) {
            ServerHttpResponse response = exchange.getResponse();
            response.setStatusCode(HttpStatus.OK);
            // 改变响应的类型
            response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
            String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.NO_LOGIN));
            DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
            return response.writeWith(Mono.just(dataBuffer));
        }

        // 3、解析
        try {
            LoginUser loginUser = JwtConfig.checkJwt(authorization);
            // 查询用户信息（username - email）
            R<LoginUserDTO> r = userFeign.getUserInfo(loginUser);
            if (!"200".equals(r.getCode())){
                ServerHttpResponse response = exchange.getResponse();
                response.setStatusCode(HttpStatus.OK);
                // 改变响应的类型
                response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
                String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.SYSTEM_ERROR));
                DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
                return response.writeWith(Mono.just(dataBuffer));
            }
//            // 信息为空创建新用户
//            if(ObjectUtil.isEmpty(r.getData())){
//                r = userFeign.creatUser(loginUser);
//                if (!"200".equals(r.getCode())){
//                    ServerHttpResponse response = exchange.getResponse();
//                    response.setStatusCode(HttpStatus.OK);
//                    // 改变响应的类型
//                    response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
//                    String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.SYSTEM_ERROR));
//                    DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
//                    return response.writeWith(Mono.just(dataBuffer));
//                }
//            }

            // redie验证token是否过期
            Boolean flag = redisTemplate.hasKey(AuthEnum.USER_TOKEN_REDIS + r.getData().getUsername());
            if (null == flag || !flag) {
                ServerHttpResponse response = exchange.getResponse();
                response.setStatusCode(HttpStatus.OK);
                // 改变响应的类型
                response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
                String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.TOKEN_TIMEOUT));
                DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
                return response.writeWith(Mono.just(dataBuffer));
            }

            LoginUserDTO finalLoginUserDTO = r.getData();
            Consumer<HttpHeaders> httpHeaders = httpHeader -> {
                httpHeader.set(AuthEnum.USER_ID_HEADER, finalLoginUserDTO.getId().toString());
                httpHeader.set(AuthEnum.USER_TENANT_ID_HEADER, finalLoginUserDTO.getTenantId().toString());
                httpHeader.set(AuthEnum.USER_NAME_HEADER, finalLoginUserDTO.getUsername());
//                httpHeader.set(AuthEnum.AUTHORIZATION, authorization);
            };
            ServerHttpRequest serverHttpRequest = exchange.getRequest().mutate().headers(httpHeaders).build();
            exchange.mutate().request(serverHttpRequest).build();
            return chain.filter(exchange);
        } catch (Exception e) {
            log.error("解析失败",e);
            ServerHttpResponse response = exchange.getResponse();
            response.setStatusCode(HttpStatus.OK);
            // 改变响应的类型
            response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
            String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.TOKEN_ERROR));
            DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
            return response.writeWith(Mono.just(dataBuffer));
        }
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
